package univlyon1.recettes.actions;

import univlyon1.recettes.dao.RecetteORMDAO;
import univlyon1.recettes.modele.Ingredient;
import univlyon1.recettes.modele.Recette;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.PrintStream;

/**
 * Created by Emmanuel Coquery on 16/09/2015.
 */
public class CreateEditRecetteAction extends MenuAction {

    private Recette recette = null;

    public CreateEditRecetteAction(BufferedReader in, PrintStream out, RecetteORMDAO dao) {
        super(in, out, dao, "Crer et diter une recette", "Edition termine");
        addAction(new AfficheAction(in, out, dao));
        addAction(new EditDescriptionAction());
        addAction(new AjouteIngredientAction(in, out, dao));
        addAction(new AjouteInstructionAction());
        addAction(new QuitAction("Terminer l'dition"));
    }

    @Override
    public String run() throws IOException {
        print("Entrer le nom de la recette: ");
        String nom = read();
        recette = dao.getRecetteByNom(nom);
        if (recette == null) {
            recette = new Recette();
            recette.setNom(nom);
            dao.createRecette(recette);
        }
        super.run();
        dao.updateRecette(recette);
        return endValue;
    }

    private class AfficheAction extends AfficheRecetteAction {

        public AfficheAction(BufferedReader in, PrintStream out, RecetteORMDAO dao) {
            super(in, out, dao);
        }

        @Override
        public String getDescription() {
            return "Afficher la recette";
        }

        @Override
        public String run() throws IOException {
            return runWithAnswer(recette.getNom());
        }
    }

    private class EditDescriptionAction implements Action {

        @Override
        public String getDescription() {
            return "Changer la description";
        }

        @Override
        public String run() throws IOException {
            println("Saisir la nouvelle description");
            recette.setDescription(readMultiLines());
            return "Description modifie";
        }
    }

    private class AjouteIngredientAction extends SimpleQuestionAction {
        public AjouteIngredientAction(BufferedReader in, PrintStream out, RecetteORMDAO dao) {
            super(in, out, dao);
        }

        @Override
        protected String getQuestion() {
            return "Entrer le nom de l'ingrdient";
        }

        @Override
        protected String runWithAnswer(String answer) throws IOException {
            Ingredient ingredient = dao.getIngredientByNom(answer);
            if (ingredient == null) {
                return "Ingrdient inconnu";
            }
            print("Saisir la quantite: ");
            double quantite = readDouble();
            print("Saisir l'unit de mesure: ");
            String unite = read();
            recette.ajouteQuantite(ingredient, quantite, unite);
            return "Ingredient ajout";
        }

        @Override
        public String getDescription() {
            return "Ajouter un ingrdient";
        }
    }

    private class AjouteInstructionAction implements Action {

        @Override
        public String getDescription() {
            return "Ajouter une tape de ralisation";
        }

        @Override
        public String run() throws IOException {
            println("Dcrire l'tape");
            String instruction = readMultiLines();
            recette.ajouteInstruction(instruction);
            return "Instruction ajoute";
        }
    }
}
